@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 github3.github
 ==============
@@ -5,20 +6,23 @@ github3.github
 This module contains the main GitHub session object.
 
 """
+from __future__ import unicode_literals
 
-from json import dumps
-from requests import session
 from github3.auths import Authorization
+from github3.decorators import (requires_auth, requires_basic_auth,
+                                requires_app_credentials)
 from github3.events import Event
 from github3.gists import Gist
 from github3.issues import Issue, issue_params
-from github3.legacy import LegacyIssue, LegacyRepo, LegacyUser
 from github3.models import GitHubCore
-from github3.orgs import Organization
+from github3.orgs import Organization, Team
 from github3.repos import Repository
+from github3.search import (CodeSearchResult, IssueSearchResult,
+                            RepositorySearchResult, UserSearchResult)
+from github3.structs import SearchIterator
 from github3.users import User, Key
-from github3.decorators import requires_auth, requires_basic_auth
 from github3.notifications import Thread
+from uritemplate import URITemplate
 
 
 class GitHub(GitHubCore):
@@ -52,7 +56,7 @@ class GitHub(GitHubCore):
         elif login and password:
             self.login(login, password)
 
-    def __repr__(self):
+    def _repr(self):
         if self._session.auth:
             return '<GitHub [{0[0]}]>'.format(self._session.auth)
         return '<GitHub at 0x{0:x}>'.format(id(self))
@@ -81,14 +85,14 @@ class GitHub(GitHubCore):
             json = self._json(self._get(url), 200)
         return Authorization(json, self) if json else None
 
-    def authorize(self, login, password, scopes, note='', note_url='',
+    def authorize(self, login, password, scopes=None, note='', note_url='',
                   client_id='', client_secret=''):
         """Obtain an authorization token from the GitHub API for the GitHub
         API.
 
         :param str login: (required)
         :param str password: (required)
-        :param list scopes: (required), areas you want this token to apply to,
+        :param list scopes: (optional), areas you want this token to apply to,
             i.e., 'gist', 'user'
         :param str note: (optional), note about the authorization
         :param str note_url: (optional), url for the application
@@ -99,17 +103,23 @@ class GitHub(GitHubCore):
         :returns: :class:`Authorization <Authorization>`
         """
         json = None
-        auth = self._session.auth or (login and password)
-        if isinstance(scopes, list) and auth:
+        # TODO: Break this behaviour in 1.0 (Don't rely on self._session.auth)
+        auth = None
+        if self._session.auth:
+            auth = self._session.auth
+        elif login and password:
+            auth = (login, password)
+
+        if auth:
             url = self._build_url('authorizations')
-            data = {'scopes': scopes, 'note': note, 'note_url': note_url,
+            data = {'note': note, 'note_url': note_url,
                     'client_id': client_id, 'client_secret': client_secret}
-            if self._session.auth:
+            if scopes:
+                data['scopes'] = scopes
+
+            with self._session.temporary_basic_auth(*auth):
                 json = self._json(self._post(url, data=data), 201)
-            else:
-                ses = session()
-                ses.auth = (login, password)
-                json = self._json(ses.post(url, data=dumps(data)), 201)
+
         return Authorization(json, self) if json else None
 
     def check_authorization(self, access_token):
@@ -170,10 +180,13 @@ class GitHub(GitHubCore):
             formatted
         :param str assignee: (optional), Login of person to assign
             the issue to
-        :param str milestone: (optional), Which milestone to assign
-            the issue to
+        :param int milestone: (optional), id number of the milestone to
+            attribute this issue to (e.g. ``m`` is a :class:`Milestone
+            <github3.issues.Milestone>` object, ``m.number`` is what you pass
+            here.)
         :param list labels: (optional), List of label names.
-        :returns: :class:`Issue <github3.issues.Issue>`
+        :returns: :class:`Issue <github3.issues.Issue>` if successful, else
+            None
         """
         repo = None
         if owner and repository and title:
@@ -257,6 +270,46 @@ class GitHub(GitHubCore):
             return key.delete()
         return False  # (No coverage)
 
+    def emojis(self):
+        """Retrieves a dictionary of all of the emojis that GitHub supports.
+
+        :returns: dictionary where the key is what would be in between the
+            colons and the value is the URL to the image, e.g., ::
+
+                {
+                    '+1': 'https://github.global.ssl.fastly.net/images/...',
+                    # ...
+                }
+        """
+        url = self._build_url('emojis')
+        return self._json(self._get(url), 200)
+
+    @requires_basic_auth
+    def feeds(self):
+        """List GitHub's timeline resources in Atom format.
+
+        :returns: dictionary parsed to include URITemplates
+        """
+        url = self._build_url('feeds')
+        json = self._json(self._get(url), 200)
+        del json['ETag']
+        del json['Last-Modified']
+
+        urls = [
+            'timeline_url', 'user_url', 'current_user_public_url',
+            'current_user_url', 'current_user_actor_url',
+            'current_user_organization_url',
+            ]
+
+        for url in urls:
+            json[url] = URITemplate(json[url])
+
+        links = json.get('_links', {})
+        for d in links.values():
+            d['href'] = URITemplate(d['href'])
+
+        return json
+
     @requires_auth
     def follow(self, login):
         """Make the authenticated user follow login.
@@ -287,6 +340,8 @@ class GitHub(GitHubCore):
         """
         url = self._build_url('gitignore', 'templates', language)
         json = self._json(self._get(url), 200)
+        if not json:
+            return ''
         return json.get('source', '')
 
     def gitignore_templates(self):
@@ -352,7 +407,7 @@ class GitHub(GitHubCore):
             return repo.issue(number)
         return None
 
-    def iter_all_repos(self, number=-1, since=None, etag=None):
+    def iter_all_repos(self, number=-1, since=None, etag=None, per_page=None):
         """Iterate over every repository in the order they were created.
 
         :param int number: (optional), number of repositories to return.
@@ -361,24 +416,28 @@ class GitHub(GitHubCore):
             restarting this iteration)
         :param str etag: (optional), ETag from a previous request to the same
             endpoint
+        :param int per_page: (optional), number of repositories to list per
+            request
         :returns: generator of :class:`Repository <github3.repos.Repository>`
         """
         url = self._build_url('repositories')
-        params = {'since': since} if since else None
-        return self._iter(int(number), url, Repository, params=params,
+        return self._iter(int(number), url, Repository,
+                          params={'since': since, 'per_page': per_page},
                           etag=etag)
 
-    def iter_all_users(self, number=-1, etag=None):
+    def iter_all_users(self, number=-1, etag=None, per_page=None):
         """Iterate over every user in the order they signed up for GitHub.
 
         :param int number: (optional), number of users to return. Default: -1,
             returns all of them
         :param str etag: (optional), ETag from a previous request to the same
             endpoint
+        :param int per_page: (optional), number of users to list per request
         :returns: generator of :class:`User <github3.users.User>`
         """
         url = self._build_url('users')
-        return self._iter(int(number), url, User, etag=etag)
+        return self._iter(int(number), url, User,
+                          params={'per_page': per_page}, etag=etag)
 
     @requires_basic_auth
     def iter_authorizations(self, number=-1, etag=None):
@@ -493,7 +552,7 @@ class GitHub(GitHubCore):
 
     @requires_auth
     def iter_org_issues(self, name, filter='', state='', labels='', sort='',
-                        direction='', since='', number=-1, etag=None):
+                        direction='', since=None, number=-1, etag=None):
         """Iterate over the organnization's issues if the authenticated user
         belongs to it.
 
@@ -509,8 +568,10 @@ class GitHub(GitHubCore):
             api-default: created
         :param str direction: accepted values: ('asc', 'desc')
             api-default: desc
-        :param str since: ISO 8601 formatted timestamp, e.g.,
-            2012-05-20T23:10:27Z
+        :param since: (optional), Only issues after this date will
+            be returned. This can be a `datetime` or an ISO8601 formatted
+            date string, e.g., 2012-05-20T23:10:27Z
+        :type since: datetime or string
         :param int number: (optional), number of issues to return. Default:
             -1, returns all available issues
         :param str etag: (optional), ETag from a previous request to the same
@@ -518,18 +579,24 @@ class GitHub(GitHubCore):
         :returns: generator of :class:`Issue <github3.issues.Issue>`
         """
         url = self._build_url('orgs', name, 'issues')
+        # issue_params will handle the since parameter
         params = issue_params(filter, state, labels, sort, direction, since)
         return self._iter(int(number), url, Issue, params, etag)
 
     @requires_auth
     def iter_issues(self, filter='', state='', labels='', sort='',
-                    direction='', since='', number=-1, etag=None):
+                    direction='', since=None, number=-1, etag=None):
         """List all of the authenticated user's (and organization's) issues.
 
+        .. versionchanged:: 0.9.0
+
+            - The ``state`` parameter now accepts 'all' in addition to 'open'
+              and 'closed'.
+
         :param str filter: accepted values:
             ('assigned', 'created', 'mentioned', 'subscribed')
             api-default: 'assigned'
-        :param str state: accepted values: ('open', 'closed')
+        :param str state: accepted values: ('all', 'open', 'closed')
             api-default: 'open'
         :param str labels: comma-separated list of label names, e.g.,
             'bug,ui,@high'
@@ -537,8 +604,10 @@ class GitHub(GitHubCore):
             api-default: created
         :param str direction: accepted values: ('asc', 'desc')
             api-default: desc
-        :param str since: ISO 8601 formatted timestamp, e.g.,
-            2012-05-20T23:10:27Z
+        :param since: (optional), Only issues after this date will
+            be returned. This can be a `datetime` or an ISO8601 formatted
+            date string, e.g., 2012-05-20T23:10:27Z
+        :type since: datetime or string
         :param int number: (optional), number of issues to return.
             Default: -1 returns all issues
         :param str etag: (optional), ETag from a previous request to the same
@@ -546,19 +615,25 @@ class GitHub(GitHubCore):
         :returns: generator of :class:`Issue <github3.issues.Issue>`
         """
         url = self._build_url('issues')
+        # issue_params will handle the since parameter
         params = issue_params(filter, state, labels, sort, direction, since)
         return self._iter(int(number), url, Issue, params, etag)
 
     @requires_auth
     def iter_user_issues(self, filter='', state='', labels='', sort='',
-                         direction='', since='', number=-1, etag=None):
+                         direction='', since=None, number=-1, etag=None):
         """List only the authenticated user's issues. Will not list
         organization's issues
 
+        .. versionchanged:: 0.9.0
+
+            - The ``state`` parameter now accepts 'all' in addition to 'open'
+              and 'closed'.
+
         :param str filter: accepted values:
             ('assigned', 'created', 'mentioned', 'subscribed')
             api-default: 'assigned'
-        :param str state: accepted values: ('open', 'closed')
+        :param str state: accepted values: ('all', 'open', 'closed')
             api-default: 'open'
         :param str labels: comma-separated list of label names, e.g.,
             'bug,ui,@high'
@@ -566,8 +641,10 @@ class GitHub(GitHubCore):
             api-default: created
         :param str direction: accepted values: ('asc', 'desc')
             api-default: desc
-        :param str since: ISO 8601 formatted timestamp, e.g.,
-            2012-05-20T23:10:27Z
+        :param since: (optional), Only issues after this date will
+            be returned. This can be a `datetime` or an ISO8601 formatted
+            date string, e.g., 2012-05-20T23:10:27Z
+        :type since: datetime or string
         :param int number: (optional), number of issues to return.
             Default: -1 returns all issues
         :param str etag: (optional), ETag from a previous request to the same
@@ -575,6 +652,7 @@ class GitHub(GitHubCore):
         :returns: generator of :class:`Issue <github3.issues.Issue>`
         """
         url = self._build_url('user', 'issues')
+        # issue_params will handle the since parameter
         params = issue_params(filter, state, labels, sort, direction, since)
         return self._iter(int(number), url, Issue, params, etag)
 
@@ -585,10 +663,15 @@ class GitHub(GitHubCore):
         """List issues on owner/repository. Only owner and repository are
         required.
 
+        .. versionchanged:: 0.9.0
+
+            - The ``state`` parameter now accepts 'all' in addition to 'open'
+              and 'closed'.
+
         :param str owner: login of the owner of the repository
         :param str repository: name of the repository
         :param int milestone: None, '*', or ID of milestone
-        :param str state: accepted values: ('open', 'closed')
+        :param str state: accepted values: ('all', 'open', 'closed')
             api-default: 'open'
         :param str assignee: '*' or login of the user
         :param str mentioned: login of the user
@@ -598,8 +681,10 @@ class GitHub(GitHubCore):
             api-default: created
         :param str direction: accepted values: ('asc', 'desc')
             api-default: desc
-        :param str since: ISO 8601 formatted timestamp, e.g.,
-            2012-05-20T23:10:27Z
+        :param since: (optional), Only issues after this date will
+            be returned. This can be a `datetime` or an ISO8601 formatted
+            date string, e.g., 2012-05-20T23:10:27Z
+        :type since: datetime or string
         :param int number: (optional), number of issues to return.
             Default: -1 returns all issues
         :param str etag: (optional), ETag from a previous request to the same
@@ -609,7 +694,8 @@ class GitHub(GitHubCore):
         if owner and repository:
             repo = self.repository(owner, repository)
             return repo.iter_issues(milestone, state, assignee, mentioned,
-                                    labels, sort, direction, since, number)
+                                    labels, sort, direction, since, number,
+                                    etag)
         return iter([])
 
     @requires_auth
@@ -763,6 +849,19 @@ class GitHub(GitHubCore):
 
         return self._iter(int(number), url, Repository, params, etag)
 
+    @requires_auth
+    def iter_user_teams(self, number=-1, etag=None):
+        """Gets the authenticated user's teams across all of organizations.
+
+        List all of the teams across all of the organizations to which the
+        authenticated user belongs. This method requires user or repo scope
+        when authenticating via OAuth.
+
+        :returns: generator of :class:`Team <github3.orgs.Team>` objects
+        """
+        url = self._build_url('user', 'teams')
+        return self._iter(int(number), url, Team, etag=etag)
+
     @requires_auth
     def key(self, id_num):
         """Gets the authenticated user's key specified by id_num.
@@ -776,18 +875,23 @@ class GitHub(GitHubCore):
             json = self._json(self._get(url), 200)
         return Key(json, self) if json else None
 
-    def login(self, username=None, password=None, token=None):
+    def login(self, username=None, password=None, token=None,
+              two_factor_callback=None):
         """Logs the user into GitHub for protected API calls.
 
-        :param str username: (optional)
-        :param str password: (optional)
-        :param str token: (optional)
+        :param str username: login name
+        :param str password: password for the login
+        :param str token: OAuth token
+        :param func two_factor_callback: (optional), function you implement to
+            provide the Two Factor Authentication code to GitHub when necessary
         """
         if username and password:
-            self._session.auth = (username, password)
+            self._session.basic_auth(username, password)
         elif token:
-            self._session.headers.update({
-                'Authorization': 'token ' + token})
+            self._session.token_auth(token)
+
+        # The Session method handles None for free.
+        self._session.two_factor_auth_callback(two_factor_callback)
 
     def markdown(self, text, mode='', context='', raw=False):
         """Render an arbitrary markdown document.
@@ -828,8 +932,9 @@ class GitHub(GitHubCore):
         return ''  # (No coverage)
 
     def meta(self):
-        """Returns an array of addresses in CIDR format specifying the
-        addresses that the incoming service hooks will originate from.
+        """Returns a dictionary with arrays of addresses in CIDR format
+        specifying theaddresses that the incoming service hooks will originate
+        from.
 
         .. versionadded:: 0.5
         """
@@ -845,6 +950,7 @@ class GitHub(GitHubCore):
         url = self._build_url('octocat')
         req = self._get(url, params={'s': say})
         return req.content if req.ok else ''
+        # TODO: Switch to req.text. Unicode is better.
 
     def organization(self, login):
         """Returns a Organization object for the login name
@@ -870,7 +976,7 @@ class GitHub(GitHubCore):
         :returns: bool
         """
         from re import match
-        m = match('https://github\.com/\w+/[\w\._-]+/events/\w+', topic)
+        m = match('https://[\w\d\-\.\:]+/\w+/[\w\._-]+/events/\w+', topic)
         status = False
         if mode and topic and callback and m:
             data = [('hub.mode', mode), ('hub.topic', topic),
@@ -896,6 +1002,29 @@ class GitHub(GitHubCore):
         r = self.repository(owner, repository)
         return r.pull_request(number) if r else None
 
+    def rate_limit(self):
+        """Returns a dictionary with information from /rate_limit.
+
+        The dictionary has two keys: ``resources`` and ``rate``. In
+        ``resources`` you can access information about ``core`` or ``search``.
+
+        Note: the ``rate`` key will be deprecated before version 3 of the
+        GitHub API is finalized. Do not rely on that key. Instead, make your
+        code future-proof by using ``core`` in ``resources``, e.g.,
+
+        ::
+
+            rates = g.rate_limit()
+            rates['resources']['core']  # => your normal ratelimit info
+            rates['resources']['search']  # => your search ratelimit info
+
+        .. versionadded:: 0.8
+
+        :returns: dict
+        """
+        url = self._build_url('rate_limit')
+        return self._json(self._get(url), 200)
+
     def repository(self, owner, repository):
         """Returns a Repository object for the specified combination of
         owner and repository
@@ -910,77 +1039,281 @@ class GitHub(GitHubCore):
             json = self._json(self._get(url), 200)
         return Repository(json, self) if json else None
 
-    def search_issues(self, owner, repo, state, keyword, start_page=0):
-        """Find issues by state and keyword.
+    @requires_app_credentials
+    def revoke_authorization(self, access_token):
+        """Revoke specified authorization for an OAuth application.
 
-        :param str owner: (required)
-        :param str repo: (required)
-        :param str state: (required), accepted values: ('open', 'closed')
-        :param str keyword: (required), what to search for
-        :param int start_page: (optional), page to get (results come 100/page)
-        :returns: list of :class:`LegacyIssue <github3.legacy.LegacyIssue>`\ s
+        Revoke all authorization tokens created by your application. This will
+        only work if you have already called ``set_client_id``.
+
+        :param str access_token: (required), the access_token to revoke
+        :returns: bool -- True if successful, False otherwise
         """
-        params = {'start_page': int(start_page)} if int(start_page) > 0 else {}
-        url = self._build_url('legacy', 'issues', 'search', owner, repo,
-                              state, keyword)
-        json = self._json(self._get(url, params=params), 200)
-        issues = json.get('issues', [])
-        return [LegacyIssue(l, self) for l in issues]
+        client_id, client_secret = self._session.retrieve_client_credentials()
+        url = self._build_url('applications', str(client_id), 'tokens',
+                              access_token)
+        with self._session.temporary_basic_auth(client_id, client_secret):
+            response = self._delete(url, params={'client_id': None,
+                                                 'client_secret': None})
 
-    def search_repos(self, keyword, language=None, start_page=None,
-                     sort=None, order=None):
-        """Search all repositories by keyword.
+        return self._boolean(response, 204, 404)
 
-        :param str keyword: (required)
-        :param str language: (optional), language to filter by
-        :param int start_page: (optional), page to get (results come 100/page)
-        :param str sort: (optional), how to sort the results; accepted values:
-            ('stars', 'forks', 'updated')
-        :param str order: (optional), sort order if ``sort`` isn't provided,
-            accepted values: ('asc', 'desc')
-        :returns: list of :class:`LegacyRepo <github3.legacy.LegacyRepo>`\ s
+    @requires_app_credentials
+    def revoke_authorizations(self):
+        """Revoke all authorizations for an OAuth application.
+
+        Revoke all authorization tokens created by your application. This will
+        only work if you have already called ``set_client_id``.
+
+        :param str client_id: (required), the client_id of your application
+        :returns: bool -- True if successful, False otherwise
         """
-        url = self._build_url('legacy', 'repos', 'search', keyword)
-        params = {'language': language, 'start_page': start_page}
+        client_id, client_secret = self._session.retrieve_client_credentials()
+        url = self._build_url('applications', str(client_id), 'tokens')
+        with self._session.temporary_basic_auth(client_id, client_secret):
+            response = self._delete(url, params={'client_id': None,
+                                                 'client_secret': None})
+
+        return self._boolean(response, 204, 404)
+
+    def search_code(self, query, sort=None, order=None, per_page=None,
+                    text_match=False, number=-1, etag=None):
+        """Find code via the code search API.
+
+        The query can contain any combination of the following supported
+        qualifiers:
+
+        - ``in`` Qualifies which fields are searched. With this qualifier you
+          can restrict the search to just the file contents, the file path, or
+          both.
+        - ``language`` Searches code based on the language it’s written in.
+        - ``fork`` Specifies that code from forked repositories should be
+          searched.  Repository forks will not be searchable unless the fork
+          has more stars than the parent repository.
+        - ``size`` Finds files that match a certain size (in bytes).
+        - ``path`` Specifies the path that the resulting file must be at.
+        - ``extension`` Matches files with a certain extension.
+        - ``user`` or ``repo`` Limits searches to a specific user or
+          repository.
+
+        For more information about these qualifiers, see: http://git.io/-DvAuA
+
+        :param str query: (required), a valid query as described above, e.g.,
+            ``addClass in:file language:js repo:jquery/jquery``
+        :param str sort: (optional), how the results should be sorted;
+            option(s): ``indexed``; default: best match
+        :param str order: (optional), the direction of the sorted results,
+            options: ``asc``, ``desc``; default: ``desc``
+        :param int per_page: (optional)
+        :param bool text_match: (optional), if True, return matching search
+            terms. See http://git.io/iRmJxg for more information
+        :param int number: (optional), number of repositories to return.
+            Default: -1, returns all available repositories
+        :param str etag: (optional), previous ETag header value
+        :return: generator of :class:`CodeSearchResult
+            <github3.search.CodeSearchResult>`
+        """
+        params = {'q': query}
+        headers = {}
+
+        if sort == 'indexed':
+            params['sort'] = sort
+
+        if sort and order in ('asc', 'desc'):
+            params['order'] = order
+
+        if text_match:
+            headers = {
+                'Accept': 'application/vnd.github.v3.full.text-match+json'
+                }
+
+        url = self._build_url('search', 'code')
+        return SearchIterator(number, url, CodeSearchResult, self, params,
+                              etag, headers)
+
+    def search_issues(self, query, sort=None, order=None, per_page=None,
+                      text_match=False, number=-1, etag=None):
+        """Find issues by state and keyword
+
+        The query can contain any combination of the following supported
+        qualifers:
+
+        - ``type`` With this qualifier you can restrict the search to issues
+          or pull request only.
+        - ``in`` Qualifies which fields are searched. With this qualifier you
+          can restrict the search to just the title, body, comments, or any
+          combination of these.
+        - ``author`` Finds issues created by a certain user.
+        - ``assignee`` Finds issues that are assigned to a certain user.
+        - ``mentions`` Finds issues that mention a certain user.
+        - ``commenter`` Finds issues that a certain user commented on.
+        - ``involves`` Finds issues that were either created by a certain user,
+          assigned to that user, mention that user, or were commented on by
+          that user.
+        - ``state`` Filter issues based on whether they’re open or closed.
+        - ``labels`` Filters issues based on their labels.
+        - ``language`` Searches for issues within repositories that match a
+          certain language.
+        - ``created`` or ``updated`` Filters issues based on times of creation,
+          or when they were last updated.
+        - ``comments`` Filters issues based on the quantity of comments.
+        - ``user`` or ``repo`` Limits searches to a specific user or
+          repository.
+
+        For more information about these qualifiers, see: http://git.io/d1oELA
+
+        :param str query: (required), a valid query as described above, e.g.,
+            ``windows label:bug``
+        :param str sort: (optional), how the results should be sorted;
+            options: ``created``, ``comments``, ``updated``;
+            default: best match
+        :param str order: (optional), the direction of the sorted results,
+            options: ``asc``, ``desc``; default: ``desc``
+        :param int per_page: (optional)
+        :param bool text_match: (optional), if True, return matching search
+          terms. See http://git.io/QLQuSQ for more information
+        :param int number: (optional), number of issues to return.
+            Default: -1, returns all available issues
+        :param str etag: (optional), previous ETag header value
+        :return: generator of :class:`IssueSearchResult
+            <github3.search.IssueSearchResult>`
+        """
+        params = {'q': query}
+        headers = {}
+
+        if sort in ('comments', 'created', 'updated'):
+            params['sort'] = sort
+
+        if order in ('asc', 'desc'):
+            params['order'] = order
+
+        if text_match:
+            headers = {
+                'Accept': 'application/vnd.github.v3.full.text-match+json'
+                }
+
+        url = self._build_url('search', 'issues')
+        return SearchIterator(number, url, IssueSearchResult, self, params,
+                              etag, headers)
+
+    def search_repositories(self, query, sort=None, order=None,
+                            per_page=None, text_match=False, number=-1,
+                            etag=None):
+        """Find repositories via various criteria.
+
+        The query can contain any combination of the following supported
+        qualifers:
+
+        - ``in`` Qualifies which fields are searched. With this qualifier you
+          can restrict the search to just the repository name, description,
+          readme, or any combination of these.
+        - ``size`` Finds repositories that match a certain size (in
+          kilobytes).
+        - ``forks`` Filters repositories based on the number of forks, and/or
+          whether forked repositories should be included in the results at
+          all.
+        - ``created`` or ``pushed`` Filters repositories based on times of
+          creation, or when they were last updated. Format: ``YYYY-MM-DD``.
+          Examples: ``created:<2011``, ``pushed:<2013-02``,
+          ``pushed:>=2013-03-06``
+        - ``user`` or ``repo`` Limits searches to a specific user or
+          repository.
+        - ``language`` Searches repositories based on the language they're
+          written in.
+        - ``stars`` Searches repositories based on the number of stars.
+
+        For more information about these qualifiers, see: http://git.io/4Z8AkA
+
+        :param str query: (required), a valid query as described above, e.g.,
+            ``tetris language:assembly``
+        :param str sort: (optional), how the results should be sorted;
+            options: ``stars``, ``forks``, ``updated``; default: best match
+        :param str order: (optional), the direction of the sorted results,
+            options: ``asc``, ``desc``; default: ``desc``
+        :param int per_page: (optional)
+        :param bool text_match: (optional), if True, return matching search
+            terms. See http://git.io/4ct1eQ for more information
+        :param int number: (optional), number of repositories to return.
+            Default: -1, returns all available repositories
+        :param str etag: (optional), previous ETag header value
+        :return: generator of :class:`Repository <github3.repos.Repository>`
+        """
+        params = {'q': query}
+        headers = {}
+
         if sort in ('stars', 'forks', 'updated'):
             params['sort'] = sort
-        if order in ('asc', 'desc') and not sort:
+
+        if order in ('asc', 'desc'):
             params['order'] = order
-        json = self._json(self._get(url, params=params), 200)
-        repos = json.get('repositories', [])
-        return [LegacyRepo(r, self) for r in repos]
 
-    def search_users(self, keyword, start_page=0, sort=None, order=None):
-        """Search all users by keyword.
+        if text_match:
+            headers = {
+                'Accept': 'application/vnd.github.v3.full.text-match+json'
+                }
 
-        :param str keyword: (required)
-        :param int start_page: (optional), page to get (results come 100/page)
-        :param str sort: (optional), how to sort the results; accepted values:
-            ('followers', 'joined', 'repositories')
-        :param str order: (optional), sort order if ``sort`` isn't provided,
-            accepted values: ('asc', 'desc')
-        :returns: list of :class:`LegacyUser <github3.legacy.LegacyUser>`\ s
+        url = self._build_url('search', 'repositories')
+        return SearchIterator(number, url, RepositorySearchResult, self,
+                              params, etag, headers)
+
+    def search_users(self, query, sort=None, order=None, per_page=None,
+                     text_match=False, number=-1, etag=None):
+        """Find users via the Search API.
+
+        The query can contain any combination of the following supported
+        qualifers:
+
+
+        - ``type`` With this qualifier you can restrict the search to just
+          personal accounts or just organization accounts.
+        - ``in`` Qualifies which fields are searched. With this qualifier you
+          can restrict the search to just the username, public email, full
+          name, or any combination of these.
+        - ``repos`` Filters users based on the number of repositories they
+          have.
+        - ``location`` Filter users by the location indicated in their
+          profile.
+        - ``language`` Search for users that have repositories that match a
+          certain language.
+        - ``created`` Filter users based on when they joined.
+        - ``followers`` Filter users based on the number of followers they
+          have.
+
+        For more information about these qualifiers see: http://git.io/wjVYJw
+
+        :param str query: (required), a valid query as described above, e.g.,
+            ``tom repos:>42 followers:>1000``
+        :param str sort: (optional), how the results should be sorted;
+            options: ``followers``, ``repositories``, or ``joined``; default:
+            best match
+        :param str order: (optional), the direction of the sorted results,
+            options: ``asc``, ``desc``; default: ``desc``
+        :param int per_page: (optional)
+        :param bool text_match: (optional), if True, return matching search
+            terms. See http://git.io/_V1zRwa for more information
+        :param int number: (optional), number of search results to return;
+            Default: -1 returns all available
+        :param str etag: (optional), ETag header value of the last request.
+        :return: generator of :class:`UserSearchResult
+            <github3.search.UserSearchResult>`
         """
-        params = {'start_page': int(start_page)} if int(start_page) > 0 else {}
-        if sort in ('followers', 'joined', 'repositories'):
+        params = {'q': query}
+        headers = {}
+
+        if sort in ('followers', 'repositories', 'joined'):
             params['sort'] = sort
-        if order in ('asc', 'desc') and not sort:
+
+        if order in ('asc', 'desc'):
             params['order'] = order
-        url = self._build_url('legacy', 'user', 'search', str(keyword))
-        json = self._json(self._get(url, params=params), 200)
-        users = json.get('users', [])
-        return [LegacyUser(u, self) for u in users]
 
-    def search_email(self, email):
-        """Search users by email.
+        if text_match:
+            headers = {
+                'Accept': 'application/vnd.github.v3.full.text-match+json'
+                }
 
-        :param str email: (required)
-        :returns: :class:`LegacyUser <github3.legacy.LegacyUser>`
-        """
-        url = self._build_url('legacy', 'user', 'email', email)
-        json = self._json(self._get(url), 200)
-        u = json.get('user', {})
-        return LegacyUser(u, self) if u else None
+        url = self._build_url('search', 'users')
+        return SearchIterator(number, url, UserSearchResult, self, params,
+                              etag, headers)
 
     def set_client_id(self, id, secret):
         """Allows the developer to set their client_id and client_secret for
@@ -1129,9 +1462,9 @@ class GitHubEnterprise(GitHub):
     """
     def __init__(self, url, login='', password='', token=''):
         super(GitHubEnterprise, self).__init__(login, password, token)
-        self._github_url = url.rstrip('/') + '/api/v3'
+        self._session.base_url = url.rstrip('/') + '/api/v3'
 
-    def __repr__(self):
+    def _repr(self):
         return '<GitHub Enterprise [0.url]>'.format(self)
 
     @requires_auth
@@ -1158,9 +1491,9 @@ class GitHubStatus(GitHubCore):
     """
     def __init__(self):
         super(GitHubStatus, self).__init__({})
-        self._github_url = 'https://status.github.com'
+        self._session.base_url = 'https://status.github.com'
 
-    def __repr__(self):
+    def _repr(self):
         return '<GitHub Status>'
 
     def _recipe(self, *args):
