@@ -1,3 +1,6 @@
+# -*- coding: utf-8 -*-
+from __future__ import unicode_literals
+
 from re import match
 from json import dumps
 from github3.decorators import requires_auth
@@ -7,9 +10,11 @@ from github3.issues.label import Label
 from github3.issues.milestone import Milestone
 from github3.models import GitHubCore
 from github3.users import User
+from uritemplate import URITemplate
 
 
 class Issue(GitHubCore):
+
     """The :class:`Issue <Issue>` object. It structures and handles the data
     returned via the `Issues <http://developer.github.com/v3/issues>`_ section
     of the GitHub API.
@@ -25,11 +30,12 @@ class Issue(GitHubCore):
         i1.id != i2.id
 
     """
+
     def __init__(self, issue, session=None):
         super(Issue, self).__init__(issue, session)
         self._api = issue.get('url', '')
         #: :class:`User <github3.users.User>` representing the user the issue
-        #  was assigned to.
+        #: was assigned to.
         self.assignee = issue.get('assignee')
         if self.assignee:
             self.assignee = User(issue.get('assignee'), self._session)
@@ -42,22 +48,28 @@ class Issue(GitHubCore):
 
         # If an issue is still open, this field will be None
         #: datetime object representing when the issue was closed.
-        self.closed_at = None
-        if issue.get('closed_at'):
-            self.closed_at = self._strptime(issue.get('closed_at'))
+        self.closed_at = self._strptime(issue.get('closed_at'))
 
         #: Number of comments on this issue.
         self.comments = issue.get('comments')
+        #: Comments url (not a template)
+        self.comments_url = issue.get('comments_url')
         #: datetime object representing when the issue was created.
         self.created_at = self._strptime(issue.get('created_at'))
+        #: Events url (not a template)
+        self.events_url = issue.get('events_url')
         #: URL to view the issue at GitHub.
         self.html_url = issue.get('html_url')
         #: Unique ID for the issue.
         self.id = issue.get('id')
-        #: Returns the list of :class:`Label <Label>`\ s on this issue.
+        #: Returns the list of :class:`Label <github3.issues.label.Label>`\ s
+        #: on this issue.
         self.labels = [Label(l, self._session) for l in issue.get('labels')]
-
-        #: :class:`Milestone <Milestone>` this issue was assigned to.
+        labels_url = issue.get('labels_url')
+        #: Labels URL Template. Expand with ``name``
+        self.labels_urlt = URITemplate(labels_url) if labels_url else None
+        #: :class:`Milestone <github3.issues.milestone.Milestone>` this
+        #: issue was assigned to.
         self.milestone = None
         if issue.get('milestone'):
             self.milestone = Milestone(issue.get('milestone'), self._session)
@@ -65,7 +77,8 @@ class Issue(GitHubCore):
         self.number = issue.get('number')
         #: Dictionary URLs for the pull request (if they exist)
         self.pull_request = issue.get('pull_request')
-        m = match('https://github\.com/(\S+)/(\S+)/issues/\d+', self.html_url)
+        m = match('https://[\w\d\-\.\:]+/(\S+)/(\S+)/(?:issues|pull)/\d+',
+                  self.html_url)
         #: Returns ('owner', 'repository') this issue was filed on.
         self.repository = m.groups()
         #: State of the issue, e.g., open, closed
@@ -75,15 +88,13 @@ class Issue(GitHubCore):
         #: datetime object representing the last time the issue was updated.
         self.updated_at = self._strptime(issue.get('updated_at'))
         #: :class:`User <github3.users.User>` who opened the issue.
-        self.user = User(issue.get('user'), self._session)
+        self.user = User(issue.get('user'), self)
 
-    def __eq__(self, other):
-        return self.id == other.id
+        closed_by = issue.get('closed_by')
+        #: :class:`User <github3.users.User>` who closed the issue.
+        self.closed_by = User(closed_by, self) if closed_by else None
 
-    def __ne__(self, other):
-        return self.id != other.id
-
-    def __repr__(self):
+    def _repr(self):
         return '<Issue [{r[0]}/{r[1]} #{n}]>'.format(r=self.repository,
                                                      n=self.number)
 
@@ -136,7 +147,7 @@ class Issue(GitHubCore):
         sigmavirus24/Todo.txt-python, the first comment's id is 4150787.
 
         :param int id_num: (required), comment id, see example above
-        :returns: :class:`IssueComment <IssueComment>`
+        :returns: :class:`IssueComment <github3.issues.comment.IssueComment>`
         """
         json = None
         if int(id_num) > 0:  # Might as well check that it's positive
@@ -151,7 +162,7 @@ class Issue(GitHubCore):
         """Create a comment on this issue.
 
         :param str body: (required), comment body
-        :returns: :class:`IssueComment <IssueComment>`
+        :returns: :class:`IssueComment <github3.issues.comment.IssueComment>`
         """
         json = None
         if body:
@@ -171,7 +182,7 @@ class Issue(GitHubCore):
             to
         :param str state: accepted values: ('open', 'closed')
         :param int milestone: the NUMBER (not title) of the milestone to
-            assign this to [1]_
+            assign this to [1]_, or 0 to remove the milestone
         :param list labels: list of labels to apply this to
         :returns: bool
 
@@ -183,6 +194,8 @@ class Issue(GitHubCore):
                 'state': state, 'milestone': milestone, 'labels': labels}
         self._remove_none(data)
         if data:
+            if 'milestone' in data and data['milestone'] == 0:
+                data['milestone'] = None
             json = self._json(self._patch(self._api, data=dumps(data)), 200)
         if json:
             self._update_(json)
@@ -202,7 +215,8 @@ class Issue(GitHubCore):
         """Iterate over the comments on this issue.
 
         :param int number: (optional), number of comments to iterate over
-        :returns: iterator of :class:`IssueComment <IssueComment>`
+        :returns: iterator of
+            :class:`IssueComment <github3.issues.comment.IssueComment>`\ s
         """
         url = self._build_url('comments', base_url=self._api)
         return self._iter(int(number), url, IssueComment)
@@ -212,11 +226,24 @@ class Issue(GitHubCore):
 
         :param int number: (optional), number of events to return. Default: -1
             returns all events available.
-        :returns: generator of :class:`IssueEvent <IssueEvent>`\ s
+        :returns: generator of
+            :class:`IssueEvent <github3.issues.event.IssueEvent>`\ s
         """
         url = self._build_url('events', base_url=self._api)
         return self._iter(int(number), url, IssueEvent)
 
+    def iter_labels(self, number=-1, etag=None):
+        """Iterate over the labels associated with this issue.
+
+        :param int number: (optional), number of labels to return. Default: -1
+            returns all labels applied to this issue.
+        :param str etag: (optional), ETag from a previous request to the same
+            endpoint
+        :returns: generator of :class:`Label <github3.issues.label.Label>`\ s
+        """
+        url = self._build_url('labels', base_url=self._api)
+        return self._iter(int(number), url, Label, etag=etag)
+
     @requires_auth
     def remove_label(self, name):
         """Removes label ``name`` from this issue.
