@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+from __future__ import unicode_literals
+
 from datetime import datetime
 from github3.models import GitHubCore
 from github3.users import User
@@ -36,5 +38,5 @@ class ContributorStats(GitHubCore):
         #: 'd' -> 'Number of deletions', 'c' -> 'Number of commits'
         self.alt_weeks = [alternate_week(w) for w in self.weeks]
 
-    def __repr__(self):
+    def _repr(self):
         return '<Contributor Statistics [{0}]>'.format(self.author)
