@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 github3.users
 =============
@@ -5,8 +6,10 @@ github3.users
 This module contains everything relating to Users.
 
 """
+from __future__ import unicode_literals
 
 from json import dumps
+from uritemplate import URITemplate
 from github3.events import Event
 from github3.models import GitHubObject, GitHubCore, BaseAccount
 from github3.decorators import requires_auth
@@ -36,18 +39,12 @@ class Key(GitHubCore):
         #: The title the user gave to the key
         self.title = key.get('title')
 
-    def __repr__(self):
+    def _repr(self):
         return '<User Key [{0}]>'.format(self.title)
 
     def __str__(self):
         return self.key
 
-    def __eq__(self, other):
-        return self.id == other.id
-
-    def __ne__(self, other):
-        return self.id != other.id
-
     def _update_(self, key):
         self.__init__(key, self._session)
 
@@ -91,7 +88,7 @@ class Plan(GitHubObject):
         #: Space allowed
         self.space = plan.get('space')
 
-    def __repr__(self):
+    def _repr(self):
         return '<Plan [{0}]>'.format(self.name)  # (No coverage)
 
     def __str__(self):
@@ -106,6 +103,7 @@ class Plan(GitHubObject):
 
 
 class User(BaseAccount):
+
     """The :class:`User <User>` object. This handles and structures information
     in the `User section <http://developer.github.com/v3/users/>`_.
 
@@ -118,7 +116,9 @@ class User(BaseAccount):
 
         u1.id == u2.id
         u1.id != u2.id
+
     """
+
     def __init__(self, user, session=None):
         super(User, self).__init__(user, session)
         if not self.type:
@@ -147,6 +147,39 @@ class User(BaseAccount):
         #: Which plan this user is on
         self.plan = Plan(user.get('plan', {}))
 
+        events_url = user.get('events_url', '')
+        #: Events URL Template. Expands with ``privacy``
+        self.events_urlt = URITemplate(events_url) if events_url else None
+
+        #: Followers URL (not a template)
+        self.followers_url = user.get('followers_url', '')
+
+        furl = user.get('following_url', '')
+        #: Following URL Template. Expands with ``other_user``
+        self.following_urlt = URITemplate(furl) if furl else None
+
+        gists_url = user.get('gists_url', '')
+        #: Gists URL Template. Expands with ``gist_id``
+        self.gists_urlt = URITemplate(gists_url) if gists_url else None
+
+        #: Organizations URL (not a template)
+        self.organizations_url = user.get('organizations_url', '')
+
+        #: Received Events URL (not a template)
+        self.received_events_url = user.get('received_events_url', '')
+
+        #: Repostories URL (not a template)
+        self.repos_url = user.get('repos_url', '')
+
+        starred_url = user.get('starred_url', '')
+        #: Starred URL Template. Expands with ``owner`` and ``repo``
+        self.starred_urlt = URITemplate(starred_url) if starred_url else None
+
+        #: Subscriptions URL (not a template)
+        self.subscriptions_url = user.get('subscriptions_url', '')
+
+        self._uniq = user.get('id', None)
+
     def __str__(self):
         return self.login
 
@@ -214,7 +247,7 @@ class User(BaseAccount):
         :returns: bool
 
         """
-        url = self._build_url('following', login, base_url=self._api)
+        url = self.following_urlt.expand(other_user=login)
         return self._boolean(self._get(url), 204, 404)
 
     def iter_events(self, public=False, number=-1, etag=None):
@@ -307,6 +340,20 @@ class User(BaseAccount):
         url = self._build_url(*path, base_url=self._api)
         return self._iter(int(number), url, Event, etag=etag)
 
+    def iter_orgs(self, number=-1, etag=None):
+        """Iterate over organizations the user is member of
+
+        :param int number: (optional), number of organizations to return.
+            Default: -1 returns all available organization
+        :param str etag: (optional), ETag from a previous request to the same
+            endpoint
+        :returns: list of :class:`Event <github3.orgs.Organization>`\ s
+        """
+        # Import here, because a toplevel import causes an import loop
+        from github3.orgs import Organization
+        url = self._build_url('orgs', base_url=self._api)
+        return self._iter(int(number), url, Organization, etag=etag)
+
     def iter_starred(self, sort=None, direction=None, number=-1, etag=None):
         """Iterate over repositories starred by this user.
 
@@ -328,7 +375,7 @@ class User(BaseAccount):
 
         params = {'sort': sort, 'direction': direction}
         self._remove_none(params)
-        url = self._build_url('starred', base_url=self._api)
+        url = self.starred_urlt.expand(owner=None, repo=None)
         return self._iter(int(number), url, Repository, params, etag)
 
     def iter_subscriptions(self, number=-1, etag=None):
